/**
 * Copyright (C) 2003-2005 Funambol
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA 
 */

package sync4j.examples.engine.source;

import java.security.Principal;
import java.sql.Timestamp;

import sync4j.framework.engine.source.*;
import sync4j.framework.engine.*;

/**
 * This class implements a dummy <i>SyncSource</i> that just displays the calls
 * to its methods
 *
 * @author  Stefano Fornari
 *
 * @version $Id: DummySyncSource.java,v 1.3 2005/04/25 07:42:42 harrie Exp $
 *
 */
public class DummySyncSource
extends AbstractSyncSource
implements SyncSource {

    private String name       = null;
    private String type       = null;
    private String sourceURI  = null;

    private SyncItem[] allItems     = null;
    private SyncItem[] newItems     = null;
    private SyncItem[] deletedItems = null;
    private SyncItem[] updatedItems = null;

    // ------------------------------------------------------------ Constructors

    /** Creates a new instance of AbstractSyncSource */
    public DummySyncSource() {
        newItems     = new SyncItem[] {
                           createItem("10",
                                      "This is a new item",
                                      SyncItemState.NEW)
                       };
        deletedItems = new SyncItem[] {
                           createItem("20",
                                      "This is a deleted item",
                                      SyncItemState.DELETED)
                       };
        updatedItems = new SyncItem[] {
                           createItem("30",
                                      "This is an UPDATED item",
                                      SyncItemState.UPDATED)
                       };

        allItems = new SyncItem[newItems.length + updatedItems.length + 1];

        allItems[0] = createItem("40",
                                 "This is an unchanged item",
                                 SyncItemState.SYNCHRONIZED);
        allItems[1] = newItems[0];
        allItems[2] = updatedItems[0];
    }

    // ---------------------------------------------------------- Public methods

    /**
     * Returns a string representation of this object.
     *
     * @return a string representation of this object.
     */
    public String toString() {
        StringBuffer sb = new StringBuffer(super.toString());

        sb.append(" - {name: ").append(getName()     );
        sb.append(" type: "   ).append(getType()     );
        sb.append(" uri: "    ).append(getSourceURI());
        sb.append("}"         );
        return sb.toString();
    }

    public SyncItem[] getAllSyncItems(Principal principal)
    throws SyncSourceException {
        System.out.println("getAllSyncItems(" + principal + ")");
        return allItems;
    }

    public SyncItem[] getDeletedSyncItems(Principal principal,
                                          Timestamp since    )
    throws SyncSourceException {
        System.out.println("getDeletedSyncItems(" +
                           principal              +
                           " , "                  +
                           since + ")");

        return deletedItems;
    }

    public SyncItemKey[] getDeletedSyncItemKeys(Principal principal,
                                                Timestamp since    )
    throws SyncSourceException {
        System.out.println("getDeletedSyncItemKeys(" +
                           principal                 +
                           " , "                     +
                           since                     +
                           ")");

        return extractKeys(deletedItems);
    }

    public SyncItem[] getNewSyncItems(Principal principal,
                                      Timestamp since    )
    throws SyncSourceException {
        System.out.println("getNewSyncItems(" +
                           principal          +
                           " , "              +
                           since              +
                           ")");

        return newItems;
    }

    public SyncItemKey[] getNewSyncItemKeys(Principal principal,
                                            Timestamp since    )
    throws SyncSourceException {
        System.out.println("getnewSyncItemKeys(" +
                           principal             +
                           " , "                 +
                           since                 +
                           ")");
        return extractKeys(newItems);
    }

    public SyncItem[] getUpdatedSyncItems(Principal principal,
                                         Timestamp since     )
    throws SyncSourceException {
        System.out.println("getUpadtedSyncItems(" +
                           principal              +
                           " , "                  +
                           since                  +
                           ")");

        return updatedItems;
    }

    public SyncItemKey[] getUpdatedSyncItemKeys(Principal principal,
                                                Timestamp since    )
    throws SyncSourceException {
        System.out.println("getUpadtedSyncItemKeys(" +
                           principal                 +
                           " , "                     +
                           since                     +
                           ")");

        return extractKeys(updatedItems);
    }

    public void removeSyncItem(Principal principal, SyncItem syncItem)
    throws SyncSourceException {
        System.out.println("removeSyncItem("                  + 
                           principal                          + 
                           " , "                              + 
                           syncItem.getKey().getKeyAsString() + 
                           ")");
    }

    public void removeSyncItems(Principal principal, SyncItem[] syncItems)
    throws SyncSourceException {
        System.out.println("removeSyncItems(" + principal + " , ...)");
        for(int i=0; i<syncItems.length; ++i) {
            removeSyncItem(principal, syncItems[i]);
        }
    }

    public SyncItem setSyncItem(Principal principal, SyncItem syncItem)
    throws SyncSourceException {
        System.out.println("setSyncItem("                     + 
                           principal                          + 
                           " , "                              + 
                           syncItem.getKey().getKeyAsString() + 
                           ")");
        return new SyncItemImpl(this, syncItem.getKey().getKeyAsString()+"-1");
    }

    public SyncItem[] setSyncItems(Principal principal, SyncItem[] syncItems)
    throws SyncSourceException {
            System.out.println("setSyncItems(" + principal + " , ...)");
            SyncItem[] ret = new SyncItem[syncItems.length];
            for (int i=0; i<syncItems.length; ++i) {
                ret[i] = setSyncItem(principal, syncItems[i]);
            }
            return ret;
    }

    public SyncItem[] getSyncItemsFromTwins(Principal principal,
                                            SyncItem[] twinItems) {
        System.out.println("getSyncItemsFromTwins(" + principal + ")");
        return new SyncItem[0];
    }

    public SyncItem getSyncItemFromTwin(Principal principal,
                                        SyncItem twinItem) {
        System.out.println("getSyncItemsFromTwin(" +
                           principal               +
                           " , ...)");
        return null;
    }

    public SyncItem getSyncItemFromId(Principal principal,
                                      SyncItemKey syncItemKey) {
        System.out.println("getSyncItemsFromId(" +
                           principal             +
                           ", "                  +
                           syncItemKey           +
                           ")");
        return null;
    }

    public SyncItem[] getSyncItemsFromIds(Principal principal,
                                          SyncItemKey[] syncItemKeys) {
        System.out.println("getSyncItemsFromIds(" + principal + ", ...)");
        return new SyncItem[0];
    }

    // ------------------------------------------------------------ Private methods

    private SyncItem createItem(String id, String content, char state) {
        SyncItem item = new SyncItemImpl(this, id, state);

        item.setProperty(
            new SyncProperty(SyncItem.PROPERTY_BINARY_CONTENT, content.getBytes())
        );

        return item;
    }

    private SyncItemKey[] extractKeys(SyncItem[] items) {
        SyncItemKey[] keys = new SyncItemKey[items.length];

        for (int i=0; i<items.length; ++i) {
            keys[i] = items[i].getKey();
        }

        return keys;
    }
}
